#######################################################################
#
#  MerlinSkinThemes for Dreambox/Enigma-2
#  Coded by marthom (c)2012 - 2014
#
#  Support: www.dreambox-tools.info
#  E-Mail: marthom@dreambox-tools.info
#
#  This plugin is open source but it is NOT free software.
#
#  This plugin may only be distributed to and executed on hardware which
#  is licensed by Dream Multimedia GmbH.
#  In other words:
#  It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
#  to hardware which is NOT licensed by Dream Multimedia GmbH.
#  It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
#  on hardware which is NOT licensed by Dream Multimedia GmbH.
#
#  If you want to use or modify the code or parts of it,
#  you have to keep MY license and inform me about the modifications by mail.
#
#######################################################################

from Plugins.Plugin import PluginDescriptor

from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.Standby import TryQuitMainloop
from Screens.ChoiceBox import ChoiceBox
from Screens.Console import Console

from skin import parseColor

from Components.ActionMap import ActionMap, HelpableActionMap
from Components.Button import Button
from Components.Input import Input
from Components.Label import Label
from Components.Sources.List import List
from Components.MenuList import MenuList
from Components.config import config, configfile, ConfigYesNo, ConfigSubsection, getConfigListEntry, ConfigSelection, ConfigNumber, ConfigText, ConfigInteger
from Components.ConfigList import ConfigListScreen
from Components.ImageCreater import imageCreater

from enigma import eListbox, eListboxPythonMultiContent, gFont, RT_HALIGN_LEFT, RT_HALIGN_CENTER, RT_HALIGN_RIGHT, RT_VALIGN_CENTER

from Tools.Directories import resolveFilename, SCOPE_SKIN, SCOPE_CURRENT_SKIN, fileExists
from Tools.Notifications import AddPopup

import xml.etree.cElementTree as Tree
from xml.dom import minidom

import socket
import struct
import base64
import shutil
import os

# =========================================
PluginVersion = "v1.7.1"
Title = "MerlinSkinThemes "
Author = "by marthom@dreambox-tools.info"
# =========================================

SkinXML = config.skin.primary_skin.value
SkinFile = resolveFilename(SCOPE_SKIN) + SkinXML
SkinName = SkinXML[0:SkinXML.find("/")]
ThemeFile = resolveFilename(SCOPE_SKIN) + SkinName + "/themes.xml"
skin_user_xml = "/etc/enigma2/skin_user.xml"

config.plugins.MerlinSkinThemes = ConfigSubsection()
config.plugins.MerlinSkinThemes.Skin = ConfigText(default=SkinName)
config.plugins.MerlinSkinThemes.selSkin = ConfigText(default=SkinName)
config.plugins.MerlinSkinThemes.ColorTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.FontTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.BorderSetTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.InfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.ChannelSelection = ConfigText(default="")
config.plugins.MerlinSkinThemes.MovieSelection = ConfigText(default="")
config.plugins.MerlinSkinThemes.MoviePlayer = ConfigText(default="")
config.plugins.MerlinSkinThemes.SecondInfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.MessageBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.InputBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.ChoiceBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.Mute = ConfigText(default="")
config.plugins.MerlinSkinThemes.Volume = ConfigText(default="")
config.plugins.MerlinSkinThemes.OLEDInfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.OLEDEventView = ConfigText(default="")
config.plugins.MerlinSkinThemes.OLEDStandby = ConfigText(default="")
config.plugins.MerlinSkinThemes.LCDInfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.LCDEventView = ConfigText(default="")
config.plugins.MerlinSkinThemes.LCDStandby = ConfigText(default="")
config.plugins.MerlinSkinThemes.PreviewScreen = ConfigText(default="")

import MerlinPreviewScreen
import MerlinSkinInfo

class MerlinSkinThemes(Screen, HelpableScreen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="1280,720" title="%s" >
			<widget name="ListLabel" position="10,10" size="575,30" font="Regular;17" zPosition="2" valign="center" halign="left" />
			<widget name="SkinsList" position="10,50" size="575,610" scrollbarMode="showOnDemand" zPosition="1" />

			<widget name="config" position="10,50" size="575,610" scrollbarMode="showOnDemand" zPosition="1" /> 
			
			<widget name="key_red" position="10,670" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_green" position="155,670" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_yellow" position="300,670" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_blue" position="445,670" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />

			<ePixmap name="red" position="10,670" zPosition="1" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="blend" />
			<ePixmap name="green" position="155,670" zPosition="1" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="blend" />
			<ePixmap name="yellow" position="300,670" zPosition="1" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="blend" />
			<ePixmap name="blue" position="445,670" zPosition="1" size="140,40" pixmap="skin_default/buttons/blue.png" transparent="1" alphatest="blend" />

			<widget name="DescLabel" position="600,10" size="670,30" font="Regular;17" zPosition="2" valign="center" halign="left" />
			<widget name="DescText" position="600,50" size="670,610" font="Regular;20" zPosition="2" halign="left" />
		</screen>"""% _("MerlinSkinThemes")

	ThemeName = ""
	selSkinName = ""
	selThemeFile = ""
	selColorThemeName = ""
	selFontThemeName = ""
	selScreenThemeName = ""
	selScreenName = ""
	
	def __init__(self, session):
		print "[MST171] running..."
		#imageCreater.createRectangle(25, 25, (255,177,77,225), "/tmp/testrect.png") 
		
		config.plugins.MerlinSkinThemes.PreviewScreen.value = "InfoBar"
		
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.clist = []
		ConfigListScreen.__init__(self, self.clist)
		
		self.setTitle(Title + " " + PluginVersion + " - " + Author)

		if not os.path.exists(skin_user_xml):
			self["ListLabel"] = Label(_("Skinlist") )
		else:
			self["ListLabel"] = Label(_("Skinlist") + " - ATTENTION: skin_user.xml found!!!")
		
		self["DescLabel"] = Label(Title + " " + PluginVersion + " " + Author)
		self["DescText"] = Label(_(
			"[OK]\nswitch to theme config screen for selected skin\n\n" + 
			"[create themes] and [delete]\ntakes some time - please wait\n\n" +
			"[INFO]\nshow skin informations (copyright, author, version, support, license)"
			))

		self.curList = "SkinsList"
		
		self["key_red"] = Button(_("exit"))
		self["key_green"] = Button(_("switch to skin"))
		self["key_yellow"] = Button(_("save as design"))
		self["key_blue"] = Button(_("skin info"))
		
		self["SkinsList"] = GetSkinsList([])

		self.onSelectionChanged = [ ]
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":     (self.exit, _("Close plugin")),
			"green":   self.buttonGreen,
			"yellow":  self.buttonYellow,
			"blue":    self.buttonBlue,
		}, -1)
		
		self["DirectionActions"] = HelpableActionMap(self, "DirectionActions",
		{
			"up":		(self.up, _("Move cursor up")),
			"down":		(self.down, _("Move cursor down")),
			"left":		(self.left, _("Move cursor left")),
			"right":	(self.right, _("Move cursor right")),
		}, -1)

		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.ok, _("")),
			"cancel":	(self.exit, _("Close plugin")),
		}, -1)

		self["TeleTextActions"] = HelpableActionMap(self, "TeleTextActions",
		{
			"info":		(self.MsgSkinInfo, _("")),
		}, -1)

		self.updateSkinsList()
		
		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		self.onLayoutFinish.append(self.startRun)

	def startRun(self):
		self["SkinsList"].onSelectionChanged.append(self.changedSkinsList)

		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		self["config"].hide()
		
		try:
			self["SkinsList"].up()
			listend = self["SkinsList"].getSelectionIndex()
			self["SkinsList"].moveToIndex(0)
			activeItem = 0
			for index in range(listend):
				self["SkinsList"].moveToIndex(index)
				if self["SkinsList"].getCurrent()[1][7] == SkinName:
					activeItem = index
				
			self["SkinsList"].moveToIndex(activeItem)
		except:
			self["SkinsList"].moveToIndex(0)

	def readThemes(self):
		defaultdesign = None
		defaultcolortheme = None
		defaultfonttheme = None
		defaultbordersettheme = None
		defaultwindowstylescrollbartheme = None
		defaultinfobar = None
		defaultchannelselection = None
		defaultmovieselection = None
		defaultmovieplayer = None
		defaultsecondinfobar = None
		defaulteventview = None
		defaultepgselection = None
		defaultmessagebox = None
		defaultinputbox = None
		defaultchoicebox = None
		defaultmute = None
		defaultvolume = None
		defaultoledinfobar = None
		defaultoledeventview = None
		defaultoledstandby = None
		defaultlcdinfobar = None
		defaultlcdeventview = None
		defaultlcdstandby = None
		
		self.clist = []

		try:
			xml = Tree.parse(MerlinSkinThemes.selThemeFile)

			selSkinList = []
			selSkinList.append(MerlinSkinThemes.selSkinName)
			config.plugins.MerlinSkinThemes.selSkin = ConfigSelection(default=MerlinSkinThemes.selSkinName, choices = selSkinList)
			self.clist.append(getConfigListEntry("Skin", config.plugins.MerlinSkinThemes.selSkin))
			
			self.clist.append(getConfigListEntry(_("----- PREVIEW ----------------------------------------------------------------"), ))
				
			PreviewList = []
			PreviewList.append("InfoBar")
			PreviewList.append("ChannelSelection")
			PreviewList.append("MovieSelection")
			PreviewList.append("SecondInfoBar")
			config.plugins.MerlinSkinThemes.PreviewScreen = ConfigSelection(default=PreviewList[0], choices = PreviewList)
			self.clist.append(getConfigListEntry("PreviewScreen", config.plugins.MerlinSkinThemes.PreviewScreen))

			self.clist.append(getConfigListEntry(_("----- DESIGNS -----------------------------------------------------------------"), ))

			# Designs
			DesignList = []
			if xml.find("designs") is not None:
				DesignList.append("-none-")
				ds = xml.find("designs")
				for design in ds.findall("design"):
					DesignList.append(design.get("name"))
					#if design.get("value") == "active":
					#	defaultdesign = design.get("name")
					defaultdesign = "-none-"

			if len(DesignList) > 0:
				self.MSTDesigns = ConfigSelection(default=defaultdesign, choices = DesignList)
				config.plugins.MerlinSkinThemes.Design = self.MSTDesigns
				self.clist.append(getConfigListEntry("Design", self.MSTDesigns))
			else:
				self.MSTDesigns = ConfigSelection(default=defaultdesign, choices = ["-none-"])
				config.plugins.MerlinSkinThemes.Design = self.MSTDesigns
				self.clist.append(getConfigListEntry("Design", self.MSTDesigns))

			self.clist.append(getConfigListEntry(_("----- THEMES -----------------------------------------------------------------"), ))

			# COLOR
			ColorThemeList = []
			if xml.find("colortheme") is not None:
				for theme in xml.findall("colortheme"):
					ColorThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultcolortheme = theme.get("name")

			if len(ColorThemeList) > 0:
				self.TColor = ConfigSelection(default=defaultcolortheme, choices = ColorThemeList)
				config.plugins.MerlinSkinThemes.ColorTheme = self.TColor
				self.clist.append(getConfigListEntry("ColorTheme", self.TColor))

			# FONT
			FontThemeList = []
			if xml.find("fonttheme") is not None:
				for theme in xml.findall("fonttheme"):
					FontThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultfonttheme = theme.get("name")

			if len(FontThemeList) > 0:
				self.TFont = ConfigSelection(default=defaultfonttheme, choices = FontThemeList)
				config.plugins.MerlinSkinThemes.FontTheme = self.TFont
				self.clist.append(getConfigListEntry("FontTheme", self.TFont))

			# BORDERSET
			BorderSetThemeList = []
			if xml.find("bordersettheme") is not None:
				for theme in xml.findall("bordersettheme"):
					BorderSetThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultbordersettheme = theme.get("name")

			if len(BorderSetThemeList) > 0:
				self.TBorder = ConfigSelection(default=defaultbordersettheme, choices = BorderSetThemeList)
				config.plugins.MerlinSkinThemes.BorderSetTheme = self.TBorder
				self.clist.append(getConfigListEntry("BorderSetTheme", self.TBorder))
				
			# WINDOSTYLESCROLLBAR
			WindowStyleScrollbarThemeList = []
			if xml.find("windowstylescrollbartheme") is not None:
				for theme in xml.findall("windowstylescrollbartheme"):
					WindowStyleScrollbarThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultwindowstylescrollbartheme = theme.get("name")

			if len(WindowStyleScrollbarThemeList) > 0:
				self.TScroll = ConfigSelection(default=defaultwindowstylescrollbartheme, choices = WindowStyleScrollbarThemeList)
				config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme = self.TScroll
				self.clist.append(getConfigListEntry("WindowStyleScrollbarTheme", self.TScroll))
			
			self.clist.append(getConfigListEntry(_("----- SCREENS ----------------------------------------------------------------"), ))

			# SCREENS
			if xml.find("screenthemes") is not None:
				#-----------
				# INFOBAR
				InfoBarList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "InfoBar":
						for themes in screens.findall("screentheme"):
							InfoBarList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultinfobar = themes.get("name")

				if len(InfoBarList) > 0:
					self.SInfobar = ConfigSelection(default=defaultinfobar, choices = InfoBarList)
					config.plugins.MerlinSkinThemes.InfoBar = self.SInfobar
					self.clist.append(getConfigListEntry("InfoBar", self.SInfobar))

				#-----------
				# CHANNELSELECTION
				ChannelSelectionList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "ChannelSelection":
						for themes in screens.findall("screentheme"):
							ChannelSelectionList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultchannelselection = themes.get("name")

				if len(ChannelSelectionList) > 0:
					self.SChannelSel = ConfigSelection(default=defaultchannelselection, choices = ChannelSelectionList)
					config.plugins.MerlinSkinThemes.ChannelSelection = self.SChannelSel
					self.clist.append(getConfigListEntry("ChannelSelection", self.SChannelSel))

				#-----------
				# MOVIESELECTION
				MovieSelectionList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MovieSelection":
						for themes in screens.findall("screentheme"):
							MovieSelectionList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmovieselection = themes.get("name")

				if len(MovieSelectionList) > 0:
					self.SMovieSel = ConfigSelection(default=defaultmovieselection, choices = MovieSelectionList)
					config.plugins.MerlinSkinThemes.MovieSelection = self.SMovieSel
					self.clist.append(getConfigListEntry("MovieSelection", self.SMovieSel))

				#-----------
				# MOVIEPLAYER
				MoviePlayerList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MoviePlayer":
						for themes in screens.findall("screentheme"):
							MoviePlayerList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmovieplayer = themes.get("name")

				if len(MoviePlayerList) > 0:
					self.SMoviePlay = ConfigSelection(default=defaultmovieplayer, choices = MoviePlayerList)
					config.plugins.MerlinSkinThemes.MoviePlayer = self.SMoviePlay		
					self.clist.append(getConfigListEntry("MoviePlayer", self.SMoviePlay))

				#-----------
				# SECONDINFOBAR
				SecondInfoBarList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "SecondInfoBar":
						for themes in screens.findall("screentheme"):
							SecondInfoBarList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultsecondinfobar = themes.get("name")

				if len(SecondInfoBarList) > 0:
					self.S2IB = ConfigSelection(default=defaultsecondinfobar, choices = SecondInfoBarList)
					config.plugins.MerlinSkinThemes.SecondInfoBar = self.S2IB
					self.clist.append(getConfigListEntry("SecondInfoBar", self.S2IB))

				#-----------
				# EventView
				EventViewList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "EventView":
						for themes in screens.findall("screentheme"):
							EventViewList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaulteventview = themes.get("name")

				if len(EventViewList) > 0:
					self.EventView = ConfigSelection(default=defaulteventview, choices = EventViewList)
					config.plugins.MerlinSkinThemes.EventView = self.EventView
					self.clist.append(getConfigListEntry("EventView", self.EventView))

				#-----------
				# EPGSelection
				EPGSelectionList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "EPGSelection":
						for themes in screens.findall("screentheme"):
							EPGSelectionList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultepgselection = themes.get("name")

				if len(EPGSelectionList) > 0:
					self.EPGSelection = ConfigSelection(default=defaultepgselection, choices = EPGSelectionList)
					config.plugins.MerlinSkinThemes.EPGSelection = self.EPGSelection
					self.clist.append(getConfigListEntry("EPGSelection", self.EPGSelection))

				#-----------
				# MESSAGEBOX
				MessageBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MessageBox":
						for themes in screens.findall("screentheme"):
							MessageBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmessagebox = themes.get("name")

				if len(MessageBoxList) > 0:
					self.SMsgBox = ConfigSelection(default=defaultmessagebox, choices = MessageBoxList)
					config.plugins.MerlinSkinThemes.MessageBox = self.SMsgBox
					self.clist.append(getConfigListEntry("MessageBox", self.SMsgBox))

				#-----------
				# INPUTBOX
				InputBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "InputBox":
						for themes in screens.findall("screentheme"):
							InputBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultinputbox = themes.get("name")

				if len(InputBoxList) > 0:
					self.SInBox = ConfigSelection(default=defaultinputbox, choices = InputBoxList)
					config.plugins.MerlinSkinThemes.InputBox = self.SInBox
					self.clist.append(getConfigListEntry("InputBox", self.SInBox))

				#-----------
				# CHOICEBOX
				ChoiceBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "ChoiceBox":
						for themes in screens.findall("screentheme"):
							ChoiceBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultchoicebox = themes.get("name")

				if len(ChoiceBoxList) > 0:
					self.SChoiceBox = ConfigSelection(default=defaultchoicebox, choices = ChoiceBoxList)
					config.plugins.MerlinSkinThemes.ChoiceBox = self.SChoiceBox
					self.clist.append(getConfigListEntry("ChoiceBox", self.SChoiceBox))
					
				#-----------
				# MUTE
				MuteList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "Mute":
						for themes in screens.findall("screentheme"):
							MuteList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmute = themes.get("name")

				if len(MuteList) > 0:
					self.SMute = ConfigSelection(default=defaultmute, choices = MuteList)
					config.plugins.MerlinSkinThemes.Mute = self.SMute
					self.clist.append(getConfigListEntry("Mute", self.SMute))
					
				#-----------
				# VOLUME
				VolumeList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "Volume":
						for themes in screens.findall("screentheme"):
							VolumeList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultvolume = themes.get("name")

				if len(VolumeList) > 0:
					self.SVolume = ConfigSelection(default=defaultvolume, choices = VolumeList)
					config.plugins.MerlinSkinThemes.Volume = self.SVolume
					self.clist.append(getConfigListEntry("Volume", self.SVolume))
					
			# LCD Screens
			#-----------
			if xml.find("lcdscreenthemes") is not None:
				self.clist.append(getConfigListEntry(_("----- LCD SCREENS ID=1 (8000, 7020HD, 800HD) 132x64 -----------------------------"), ))		
				# INFOBAR
				LCDInfoBarList = []
				st = xml.find("lcdscreenthemes")
				if st.find("screens[@name='InfoBarSummary']") is not None:
					lst = st.find("screens[@name='InfoBarSummary']")
					for th in lst.findall("lcdscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "InfoBarSummary" and screen.get("id") == "1":
								LCDInfoBarList.append(th.get("name"))
								if th.get("value") == "active":
									defaultlcdinfobar = th.get("name")

					if len(LCDInfoBarList) > 0:
						self.LSInfobar = ConfigSelection(default=defaultlcdinfobar, choices = LCDInfoBarList)
						config.plugins.MerlinSkinThemes.LCDInfoBar = self.LSInfobar
						self.clist.append(getConfigListEntry("LCDInfoBar", self.LSInfobar))

				#-----------
				# EVENTVIEW
				LCDEventViewList = []
				if st.find("screens[@name='EventView_summary']") is not None:
					lst = st.find("screens[@name='EventView_summary']")
					for th in lst.findall("lcdscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "EventView_summary" and screen.get("id") == "1":
								LCDEventViewList.append(th.get("name"))
								if th.get("value") == "active":
									defaultlcdeventview = th.get("name")

					if len(LCDEventViewList) > 0:
						self.LSEventView = ConfigSelection(default=defaultlcdeventview, choices = LCDEventViewList)
						config.plugins.MerlinSkinThemes.LCDEventView = self.LSEventView
						self.clist.append(getConfigListEntry("LCDEventView", self.LSEventView))

				#-----------
				# STANDBY
				LCDStandbyList = []
				if st.find("screens[@name='StandbySummary']") is not None:
					lst = st.find("screens[@name='StandbySummary']")
					for th in lst.findall("lcdscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "StandbySummary" and screen.get("id") == "1":
								LCDStandbyList.append(th.get("name"))
								if th.get("value") == "active":
									defaultlcdstandby = th.get("name")

					if len(LCDStandbyList) > 0:
						self.LSStandby = ConfigSelection(default=defaultlcdstandby, choices = LCDStandbyList)
						config.plugins.MerlinSkinThemes.LCDStandby = self.LSStandby
						self.clist.append(getConfigListEntry("LCDStandby", self.LSStandby))
				
			# OLED Screens
			if xml.find("oledscreenthemes") is not None:
				self.clist.append(getConfigListEntry(_("----- OLED SCREENS ID=2 (800se) 96x64 ------------------------------------------"), ))		
				#-----------
				# INFOBAR
				OLEDInfoBarList = []
				st = xml.find("oledscreenthemes")
				if st.find("screens[@name='InfoBarSummary']") is not None:
					ost = st.find("screens[@name='InfoBarSummary']")
					for th in ost.findall("oledscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "InfoBarSummary" and screen.get("id") == "2":
								OLEDInfoBarList.append(th.get("name"))
								if th.get("value") == "active":
									defaultoledinfobar = th.get("name")

					if len(OLEDInfoBarList) > 0:
						self.OSInfobar = ConfigSelection(default=defaultoledinfobar, choices = OLEDInfoBarList)
						config.plugins.MerlinSkinThemes.OLEDInfoBar = self.OSInfobar
						self.clist.append(getConfigListEntry("OLEDInfoBar", self.OSInfobar))

				#-----------
				# EVENTVIEW
				OLEDEventViewList = []
				if st.find("screens[@name='EventView_summary']") is not None:
					ost = st.find("screens[@name='EventView_summary']")
					for th in ost.findall("oledscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "EventView_summary" and screen.get("id") == "2":
								OLEDEventViewList.append(th.get("name"))
								if th.get("value") == "active":
									defaultoledeventview = th.get("name")

					if len(OLEDEventViewList) > 0:
						self.OSEventView = ConfigSelection(default=defaultoledeventview, choices = OLEDEventViewList)
						config.plugins.MerlinSkinThemes.OLEDEventView = self.OSEventView
						self.clist.append(getConfigListEntry("OLEDEventView", self.OSEventView))

				#-----------
				# STANDBY
				OLEDStandbyList = []
				if st.find("screens[@name='StandbySummary']") is not None:
					ost = st.find("screens[@name='StandbySummary']")
					for th in ost.findall("oledscreentheme"):
						for screen in th.findall("screen"):
							if screen.get("name") == "StandbySummary" and screen.get("id") == "2":
								OLEDStandbyList.append(th.get("name"))
								if th.get("value") == "active":
									defaultoledstandby = th.get("name")

					if len(OLEDStandbyList) > 0:
						self.OSStandby = ConfigSelection(default=defaultoledstandby, choices = OLEDStandbyList)
						config.plugins.MerlinSkinThemes.OLEDStandby = self.OSStandby
						self.clist.append(getConfigListEntry("OLEDStandby", self.OSStandby))
				
		except:
			print "[MST] themes.xml in " + MerlinSkinThemes.selSkinName + " corrupt!"
			self.clist.append(getConfigListEntry(_("----- ERROR - themes.xml in " + MerlinSkinThemes.selSkinName + " corrupt! -------"), ))
			
		self["config"].setList(self.clist)
		
	def buttonGreen(self):
		if self.curList == "SkinsList":
			# set new skin
			sel = self["SkinsList"].getCurrent()
			if sel[1][7] == "Default Skin":
				skinfile = "skin.xml"
			else:
				skinfile = "%s/skin.xml" % sel[1][7]

			# Dr. Best Infobar position
			if os.path.isfile("/usr/share/enigma2/merlin_setup.xml") == True:
				config.merlin2.infobar_position_offset_x.value = 0
				config.merlin2.infobar_position_offset_x.save()
				config.merlin2.infobar_position_offset_y.value = 0
				config.merlin2.infobar_position_offset_y.save()
				config.merlin2.movieplayer_infobar_position_offset_x.value = 0
				config.merlin2.movieplayer_infobar_position_offset_x.save()
				config.merlin2.movieplayer_infobar_position_offset_y.value = 0
				config.merlin2.movieplayer_infobar_position_offset_y.save()
				
			config.skin.primary_skin.value = skinfile
			config.skin.primary_skin.save()
			restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("GUI needs a restart to apply a new skin\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
			restartbox.setTitle(_("Restart GUI now?"))
		elif self.curList == "ConfigList":
			askBox = self.session.openWithCallback(self.askYN,MessageBox,_("[apply themes] needs time to build new settings\nDo you want to do this now?"), MessageBox.TYPE_YESNO)
			askBox.setTitle(_("Apply themes now?"))

	def askYN(self, answer):
		if answer is True:
			self.setThemes()
			if SkinName == MerlinSkinThemes.selSkinName:
				restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("GUI needs a restart to apply a new skin\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
				restartbox.setTitle(_("Restart GUI now?"))
			
			else:
				self.session.open(MessageBox, _("Changes to skin " + MerlinSkinThemes.selSkinName + " ready!"), MessageBox.TYPE_INFO)
		#else:
		#	self.exit()
	
	def buttonBlue(self):
		if self.curList == "SkinsList":
			reload(MerlinSkinInfo)
			self.session.open(MerlinSkinInfo.MerlinSkinInfo)
			
		else:
			# check if previewscreen exist
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			rootTheme = curTree.getroot()
			if rootTheme.find("screenthemes") is not None:
				found = False
				themes = rootTheme.find("screenthemes")
				for screens in themes.findall("screens"):
					if screens.get("name") == config.plugins.MerlinSkinThemes.PreviewScreen.value:
						found = True
						
				if found:
					reload(MerlinPreviewScreen)
					self.session.open(MerlinPreviewScreen.MerlinPreviewScreen)
				else:
					self.session.open(MessageBox,_("The screen " + config.plugins.MerlinSkinThemes.PreviewScreen.value + " could found in themes.xml"), MessageBox.TYPE_ERROR, title=_("Error"))
			else:
				self.session.open(MessageBox,_("No section <screenthemes> in themes.xml found"), MessageBox.TYPE_ERROR, title=_("Error"))
				
	def buttonYellow(self):
		if self.curList == "SkinsList":
			if self["SkinsList"].getCurrent()[3][7] == "no themes.xml":
				self.createThemes()

			if self["SkinsList"].getCurrent()[3][7] == "no skin.xml":
				self.delSkinDir()
				
		elif self.curList == "ConfigList":
			if self["config"].getCurrent()[0] == "Design":
				# delete design
				self.deleteDesign()
			
			else:
				# save as design
				self.session.openWithCallback(self.saveDesign, InputBox, title=_("Please enter designname!"))
			
	def saveDesign(self, designname):
		if designname is not None:
		
			designname = designname.strip()
			
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			xmlroot = curTree.getroot()
			
			if xmlroot.find("designs") is None:
				xmldesigns = Tree.SubElement(xmlroot, "designs")
			else:
				xmldesigns = xmlroot.find("designs")

			# check if design exists
			if xmldesigns.find("design[@name='" + designname + "']") is not None:
				xmldesigns.remove(xmldesigns.find("design[@name='" + designname + "']"))
				
			# write/rewrite design
			xmldesign = Tree.SubElement(xmldesigns, "design", {"name": designname, "value": "active"})
				
			if xmlroot.find("colortheme[@name='" + config.plugins.MerlinSkinThemes.ColorTheme.value + "']") is not None:
				if xmldesign.find("ColorTheme") is not None:
					td = xmldesign.find("ColorTheme")
					td.set("name", config.plugins.MerlinSkinThemes.ColorTheme.value)
				else:
					Tree.SubElement(xmldesign, "ColorTheme", {"name": config.plugins.MerlinSkinThemes.ColorTheme.value})
				
			if xmlroot.find("fonttheme[@name='" + config.plugins.MerlinSkinThemes.FontTheme.value + "']") is not None:
				if xmldesign.find("FontTheme") is not None:
					td = d.find("FontTheme")
					td.set("name", config.plugins.MerlinSkinThemes.FontTheme.value)
				else:
					Tree.SubElement(xmldesign, "FontTheme", {"name": config.plugins.MerlinSkinThemes.FontTheme.value})
			
			if xmlroot.find("bordersettheme") is not None:
				if xmldesign.find("BorderSetTheme") is not None:
					td = d.find("BorderSetTheme")
					td.set("name", config.plugins.MerlinSkinThemes.BorderSetTheme.value)
				else:
					Tree.SubElement(xmldesign, "BorderSetTheme", {"name": config.plugins.MerlinSkinThemes.BorderSetTheme.value})
			
			if xmlroot.find("windowstylescrollbartheme") is not None:
				if xmldesign.find("WindowStyleScrollbarTheme") is not None:
					td = d.find("WindowStyleScrollbarTheme")
					td.set("name", config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value)
				else:
					Tree.SubElement(xmldesign, "WindowStyleScrollbarTheme", {"name": config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value})
			
			# Screens
			if xmlroot.find("screenthemes") is not None:
				t = xmlroot.find("screenthemes")
				
				if t.find("screens[@name='InfoBar']") is not None:
					ts = t.find("screens[@name='InfoBar']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.InfoBar.value + "']") is not None:
						Tree.SubElement(xmldesign, "InfoBar", {"name": config.plugins.MerlinSkinThemes.InfoBar.value})
			
				if t.find("screens[@name='ChannelSelection']") is not None:
					ts = t.find("screens[@name='ChannelSelection']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.ChannelSelection.value + "']") is not None:
						Tree.SubElement(xmldesign, "ChannelSelection", {"name": config.plugins.MerlinSkinThemes.ChannelSelection.value})
						
				if t.find("screens[@name='MovieSelection']") is not None:
					ts = t.find("screens[@name='MovieSelection']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.MovieSelection.value + "']") is not None:
						Tree.SubElement(xmldesign, "MovieSelection", {"name": config.plugins.MerlinSkinThemes.MovieSelection.value})
						
				if t.find("screens[@name='MoviePlayer']") is not None:
					ts = t.find("screens[@name='MoviePlayer']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.MoviePlayer.value + "']") is not None:
						Tree.SubElement(xmldesign, "MoviePlayer", {"name": config.plugins.MerlinSkinThemes.MoviePlayer.value})
						
				if t.find("screens[@name='SecondInfoBar']") is not None:
					ts = t.find("screens[@name='SecondInfoBar']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.SecondInfoBar.value + "']") is not None:
						Tree.SubElement(xmldesign, "SecondInfoBar", {"name": config.plugins.MerlinSkinThemes.SecondInfoBar.value})
						
				if t.find("screens[@name='EventView']") is not None:
					ts = t.find("screens[@name='EventView']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.EventView.value + "']") is not None:
						Tree.SubElement(xmldesign, "EventView", {"name": config.plugins.MerlinSkinThemes.EventView.value})
						
				if t.find("screens[@name='EPGSelection']") is not None:
					ts = t.find("screens[@name='EPGSelection']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.EPGSelection.value + "']") is not None:
						Tree.SubElement(xmldesign, "EPGSelection", {"name": config.plugins.MerlinSkinThemes.EPGSelection.value})
						
				if t.find("screens[@name='MessageBox']") is not None:
					ts = t.find("screens[@name='MessageBox']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.MessageBox.value + "']") is not None:
						Tree.SubElement(xmldesign, "MessageBox", {"name": config.plugins.MerlinSkinThemes.MessageBox.value})
						
				if t.find("screens[@name='InputBox']") is not None:
					ts = t.find("screens[@name='InputBox']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.InputBox.value + "']") is not None:
						Tree.SubElement(xmldesign, "InputBox", {"name": config.plugins.MerlinSkinThemes.InputBox.value})
						
				if t.find("screens[@name='ChoiceBox']") is not None:
					ts = t.find("screens[@name='ChoiceBox']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.ChoiceBox.value + "']") is not None:
						Tree.SubElement(xmldesign, "ChoiceBox", {"name": config.plugins.MerlinSkinThemes.ChoiceBox.value})
						
				if t.find("screens[@name='Mute']") is not None:
					ts = t.find("screens[@name='Mute']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.Mute.value + "']") is not None:
						Tree.SubElement(xmldesign, "Mute", {"name": config.plugins.MerlinSkinThemes.Mute.value})
						
				if t.find("screens[@name='Volume']") is not None:
					ts = t.find("screens[@name='Volume']")
					if ts.find("screentheme[@name='" + config.plugins.MerlinSkinThemes.Volume.value + "']") is not None:
						Tree.SubElement(xmldesign, "Volume", {"name": config.plugins.MerlinSkinThemes.Volume.value})

			# LCD Screens
			if xmlroot.find("lcdscreenthemes") is not None:
				t = xmlroot.find("lcdscreenthemes")
				
				if t.find("screens[@name='InfoBarSummary']") is not None:
					ts = t.find("screens[@name='InfoBarSummary']")
					if ts.find("lcdscreentheme[@name='" + config.plugins.MerlinSkinThemes.LCDInfoBar.value + "']") is not None:
						Tree.SubElement(xmldesign, "LCDInfoBar", {"name": config.plugins.MerlinSkinThemes.LCDInfoBar.value})

				if t.find("screens[@name='EventView_summary']") is not None:
					ts = t.find("screens[@name='EventView_summary']")
					if ts.find("lcdscreentheme[@name='" + config.plugins.MerlinSkinThemes.LCDEventView.value + "']") is not None:
						Tree.SubElement(xmldesign, "LCDEventView", {"name": config.plugins.MerlinSkinThemes.LCDEventView.value})

				if t.find("screens[@name='StandbySummary']") is not None:
					ts = t.find("screens[@name='StandbySummary']")
					if ts.find("lcdscreentheme[@name='" + config.plugins.MerlinSkinThemes.LCDStandby.value + "']") is not None:
						Tree.SubElement(xmldesign, "LCDStandby", {"name": config.plugins.MerlinSkinThemes.LCDStandby.value})

			# OLED Screens
			if xmlroot.find("oledscreenthemes") is not None:
				t = xmlroot.find("oledscreenthemes")
				
				if t.find("screens[@name='InfoBarSummary']") is not None:
					ts = t.find("screens[@name='InfoBarSummary']")
					if ts.find("oledscreentheme[@name='" + config.plugins.MerlinSkinThemes.OLEDInfoBar.value + "']") is not None:
						Tree.SubElement(xmldesign, "OLEDInfoBar", {"name": config.plugins.MerlinSkinThemes.OLEDInfoBar.value})

				if t.find("screens[@name='EventView_summary']") is not None:
					ts = t.find("screens[@name='EventView_summary']")
					if ts.find("oledscreentheme[@name='" + config.plugins.MerlinSkinThemes.OLEDEventView.value + "']") is not None:
						Tree.SubElement(xmldesign, "OLEDEventView", {"name": config.plugins.MerlinSkinThemes.OLEDEventView.value})

				if t.find("screens[@name='StandbySummary']") is not None:
					ts = t.find("screens[@name='StandbySummary']")
					if ts.find("oledscreentheme[@name='" + config.plugins.MerlinSkinThemes.OLEDStandby.value + "']") is not None:
						Tree.SubElement(xmldesign, "OLEDStandby", {"name": config.plugins.MerlinSkinThemes.OLEDStandby.value})

			self.XMLindent(xmlroot, 0)
			
			curTree.write(MerlinSkinThemes.selThemeFile)
				
			self.readThemes()
				
	def deleteDesign(self):
		if self.MSTDesigns.value == "-none-":
			self.session.open(MessageBox,_("nothing to delete"), MessageBox.TYPE_ERROR)
		else:
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			xmlroot = curTree.getroot()
			designs = xmlroot.find("designs")
			for design in designs.findall("design"):
				if design.get("name") == config.plugins.MerlinSkinThemes.Design.value:
					designs.remove(design)
			
					self.XMLindent(xmlroot, 0)

					curTree.write(MerlinSkinThemes.selThemeFile)
		
					self.readThemes()
						
	def setDesign(self):
		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		xmlroot = curTree.getroot()
		designs = xmlroot.find("designs")
		for design in designs.findall("design"):
			if design.get("name") == config.plugins.MerlinSkinThemes.Design.value:
				if design.find("ColorTheme") is not None:
					tmp = design.find("ColorTheme")
					try:
						self.TColor.setValue(tmp.get("name"))
					except:
						print "[MST171] TColor not found"
					
				if design.find("FontTheme") is not None:
					tmp = design.find("FontTheme")
					try:
						self.TFont.setValue(tmp.get("name"))
					except:
						print "[MST171] TFont not found"

				if design.find("BorderSetTheme") is not None:
					tmp = design.find("BorderSetTheme")
					try:
						self.TBorder.setValue(tmp.get("name"))
					except:
						print "[MST171] TBorder not found"
					
				if design.find("WindowStyleScrollbarTheme") is not None:
					tmp = design.find("WindowStyleScrollbarTheme")
					#print "[MST] TScroll: %s" % self["config"].getCurrent()[1].getChoices()
					#print "[MST] TScroll: %s" % self["config"].getCurrent()[0]
					#print "[MST] TScroll len: %s" % len(self["config"].getCurrent()[1].getChoices())

					#print "[MST] BEVOR"
					#print "[MST] TScroll: %s" % self.TScroll.getChoices()
					#print "[MST] TScroll default: %s" % tmp.get("name")
					#print "[MST] TScroll index: %s" % self.TScroll.getChoices().index(tmp.get("name"))
					#print "[MST] TScroll len: %s" % len(self.TScroll.getChoices())
					
					try:
						self.TScroll.setValue(tmp.get("name"))
					except:
						print "[MST171] TScroll not found"
					
					#print "[MST] AFTER"
					#print "[MST] TScroll: %s" % self.TScroll.getChoices()
					#print "[MST] TScroll default: %s" % tmp.get("name")
					#print "[MST] TScroll len: %s" % len(self.TScroll.getChoices())
					
				if design.find("InfoBar") is not None:
					tmp = design.find("InfoBar")
					try:
						self.SInfobar.setValue(tmp.get("name"))
					except:
						print "[MST171] SInfobar not found"
					
				if design.find("ChannelSelection") is not None:
					tmp = design.find("ChannelSelection")
					try:
						self.SChannelSel.setValue(tmp.get("name"))
					except:
						print "[MST171] SChannelSel not found"
					
				if design.find("MovieSelection") is not None:
					tmp = design.find("MovieSelection")
					try:
						self.SMovieSel.setValue(tmp.get("name"))
					except:
						print "[MST171] SMovieSel not found"
					
				if design.find("MoviePlayer") is not None:
					tmp = design.find("MoviePlayer")
					try:
						self.SMoviePlay.setValue(tmp.get("name"))
					except:
						print "[MST171] SMoviePlay not found"
					
				if design.find("SecondInfoBar") is not None:
					tmp = design.find("SecondInfoBar")
					try:
						self.S2IB.setValue(tmp.get("name"))
					except:
						print "[MST171] S2IB not found"
					
				if design.find("EventView") is not None:
					tmp = design.find("EventView")
					try:
						self.EventView.setValue(tmp.get("name"))
					except:
						print "[MST171] EventView not found"
					
				if design.find("EPGSelection") is not None:
					tmp = design.find("EPGSelection")
					try:
						self.EPGSelection.setValue(tmp.get("name"))
					except:
						print "[MST171] EPGSelection not found"
					
				if design.find("MessageBox") is not None:
					tmp = design.find("MessageBox")
					try:
						self.SMsgBox.setValue(tmp.get("name"))
					except:
						print "[MST171] SMsgBox not found"
					
				if design.find("InputBox") is not None:
					tmp = design.find("InputBox")
					try:
						self.SInBox.setValue(tmp.get("name"))
					except:
						print "[MST171] SInBox not found"
					
				if design.find("ChoiceBox") is not None:
					tmp = design.find("ChoiceBox")
					try:
						self.SChoiceBox.setValue(tmp.get("name"))
					except:
						print "[MST171] SChoiceBox not found"

				if design.find("Mute") is not None:
					tmp = design.find("Mute")
					try:
						self.SMute.setValue(tmp.get("name"))
					except:
						print "[MST171] SMute not found"

				if design.find("Volume") is not None:
					tmp = design.find("Volume")
					try:
						self.SVolume.setValue(tmp.get("name"))
					except:
						print "[MST171] SVolume not found"

				# LCD
				if design.find("LCDInfoBar") is not None:
					tmp = design.find("LCDInfoBar")
					try:
						self.LSInfobar.setValue(tmp.get("name"))
					except:
						print "[MST171] LSInfobar not found"
					
				if design.find("LCDEventView") is not None:
					tmp = design.find("LCDEventView")
					try:
						self.LSEventView.setValue(tmp.get("name"))
					except:
						print "[MST171] LSEventView not found"
					
				if design.find("LCDStandby") is not None:
					tmp = design.find("LCDStandby")
					try:
						self.LSStandby.setValue(tmp.get("name"))
					except:
						print "[MST171] LSStandby not found"
					
				# OLED
				if design.find("OLEDInfoBar") is not None:
					tmp = design.find("OLEDInfoBar")
					try:
						self.OSInfobar.setValue(tmp.get("name"))
					except:
						print "[MST171] OSInfobar not found"
					
				if design.find("OLEDEventView") is not None:
					tmp = design.find("OLEDEventView")
					try:
						self.OSEventView.setValue(tmp.get("name"))
					except:
						print "[MST171] OSEventView not found"
					
				if design.find("OLEDStandby") is not None:
					tmp = design.find("OLEDStandby")
					try:
						self.OSStandby.setValue(tmp.get("name"))
					except:
						print "[MST171] OSStandby not found"
					
				# refresh Screen
				self["config"].setList(self.clist)
				#self["config"].hide()
				#self["config"].show()
			
	def ok(self):
		if self.curList == "SkinsList":
			if self["SkinsList"].getCurrent()[3][7] == "":
				self.curList = "ConfigList"
				
				if not os.path.exists(skin_user_xml):
					self["ListLabel"].setText(_("Configlist") )
				else:
					self["ListLabel"].setText(_("Configlist") + " - ATTENTION: skin_user.xml found!!!")

				self["DescText"].setText(_(
					"[OK]\nswitch to skin selection screen\n\n" + 
					"[apply themes] or [preview]\ntakes some time - please wait\n\n" + 
					"[INFO]\nshow skin informations (copyright, author, version, support, license)\n\n" +
					"[delete design]\ndelete selected design\n\n" + 
					"[save as design]\nselected themes/screens stored in new design\n\n" +
					"choose Design: -none-\nto reset themes to active settings"
				))

				if self["config"].getCurrent()[0] == "Design":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("delete design"))
					self["key_blue"].setText(_("preview"))
				elif self["config"].getCurrent()[0] == "Skin":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				elif self["config"].getCurrent()[0] == "PreviewScreen":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				else:
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				
				self["key_green"].show()
				self["key_yellow"].show()
				
				self["SkinsList"].hide()
				self["config"].show()
				
				config.plugins.MerlinSkinThemes.Skin.value = self["SkinsList"].getCurrent()[1][7]
			else:
				self.session.open(MessageBox,_("No themes.xml or skin.xml found.\nPlease select a valid skin including themes.xml"), MessageBox.TYPE_ERROR, title=_("Error"))
		else:
			self.curList = "SkinsList"
 			
			if not os.path.exists(skin_user_xml):
				self["ListLabel"].setText(_("Skinlist") )
			else:
				self["ListLabel"].setText(_("Skinlist") + " - ATTENTION: skin_user.xml found!!!")
				
			self["DescText"].setText(_(
				"[OK]\nswitch to theme config screen for selected skin\n\n" + 
				"[create themes] and [delete]\ntakes some time - please wait\n\n" +
				"[INFO]\nshow skin informations (copyright, author, version, support, license)"
			))

			self["key_green"].setText(_("switch to skin"))
			self["key_green"].hide()
			self["key_yellow"].setText(_(""))
			self["key_yellow"].hide()
			self["key_blue"].setText(_("skin info"))

			self.updateSkinsList()
			self["SkinsList"].show()
			self["config"].hide()

	def up(self):
		if self.curList == "SkinsList":
			self[self.curList].up()
		else:
			self["config"].instance.moveSelection(self["config"].instance.moveUp)
			if self["config"].getCurrent()[0] == "Design":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("delete design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "Skin":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "PreviewScreen":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			else:
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
	
	def down(self):
		if self.curList == "SkinsList":
			self[self.curList].down()
		else:
			self["config"].instance.moveSelection(self["config"].instance.moveDown)
			if self["config"].getCurrent()[0] == "Design":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("delete design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "Skin":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "PreviewScreen":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			else:
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
	
	def left(self):
		if self.curList == "SkinsList":
			self[self.curList].pageUp()
		else:
			ConfigListScreen.keyLeft(self)
			if self["config"].getCurrent()[0] == "Design":
				if config.plugins.MerlinSkinThemes.Design.value == "-none-":
					self.readThemes()
				else:
					self.setDesign()
	
	def right(self):
		if self.curList == "SkinsList":
			self[self.curList].pageDown()
		else:
			ConfigListScreen.keyRight(self)
			if self["config"].getCurrent()[0] == "Design":
				if config.plugins.MerlinSkinThemes.Design.value == "-none-":
					self.readThemes()
				else:
					self.setDesign()
	
	def changedSkinsList(self):
		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		if self["SkinsList"].getCurrent()[2][7] == "active skin":
			self["key_green"].hide()
		else:
			self["key_green"].show()

		if self["SkinsList"].getCurrent()[3][7] == "no skin.xml":
			self["key_green"].hide()

			self["key_yellow"].show()
			self["key_yellow"].setText(_("delete"))
			
		elif self["SkinsList"].getCurrent()[3][7] == "no themes.xml":
			self["key_green"].show()
			
			self["key_yellow"].show()
			self["key_yellow"].setText(_("create themes"))
			
		else:
			self["key_yellow"].show()

			self["key_yellow"].hide()
			self.readThemes()
			
	def updateSkinsList(self):
		self["SkinsList"].buildList()

	def setThemes(self):
		# set all "inactive", set new theme "active"
		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		rootTheme = curTree.getroot()

		curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
		rootSkin = curSkin.getroot()
		
		if rootTheme.find("colortheme") is not None:
			for theme in rootTheme.findall("colortheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.ColorTheme.value:
					theme.set("value", "active")

					# colors
					SkinColors = rootSkin.find("colors")

					# delete old colors
					for color in SkinColors.findall("color"):
						SkinColors.remove(color)
					
					# Set new colors
					ThemeColors = theme.find("colors")
					
					for color in ThemeColors.findall("color"):
						name = color.get("name")
						value = color.get("value")
						
						Tree.SubElement(SkinColors, "color", {"name": name, "value": value})
					
				else:
					theme.set("value", "inactive")

		if rootTheme.find("fonttheme") is not None:
			for theme in rootTheme.findall("fonttheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.FontTheme.value:
					theme.set("value", "active")

					# fonts
					SkinFonts = rootSkin.find("fonts")

					# delete old fonts
					for font in SkinFonts.findall("font"):
						SkinFonts.remove(font)
					
					# Set new fonts
					ThemeFonts = theme.find("fonts")
					
					for font in ThemeFonts.findall("font"):
						filename = font.get("filename")
						name = font.get("name")
						scale = font.get("scale", "100")
						replacement = font.get("replacement", "0")
						
						Tree.SubElement(SkinFonts, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})
						

				else:
					theme.set("value", "inactive")

		if rootTheme.find("bordersettheme") is not None:
			for theme in rootTheme.findall("bordersettheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.BorderSetTheme.value:
					theme.set("value", "active")

					# WindowStyle
					for ws in rootSkin.findall("windowstyle"):
						if ws.get("id") == "0":
							for bs in ws.findall("borderset"):
								if bs.get("name") == "bsWindow":
									for px in bs.findall("pixmap"):
										bs.remove(px)
										
									for tbs in theme.findall("borderset"):
										if tbs.get("name") == "bsWindow":
											for tpx in tbs.findall("pixmap"):
												bs.append(Tree.fromstring(Tree.tostring(tpx)))

								if bs.get("name") == "bsListboxEntry":
									for px in bs.findall("pixmap"):
										bs.remove(px)
						
									for tbs in theme.findall("borderset"):
										if tbs.get("name") == "bsListboxEntry":
											for tpx in tbs.findall("pixmap"):
												bs.append(Tree.fromstring(Tree.tostring(tpx)))
						else:
							print "[MST] id not found in windowstyle"
						
				else:
					theme.set("value", "inactive")
					
		if rootTheme.find("windowstylescrollbartheme") is not None:
			for theme in rootTheme.findall("windowstylescrollbartheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value:
					theme.set("value", "active")

					# WindowStyleScrollbar
					for wssb in rootSkin.findall("windowstylescrollbar"):
						if wssb.get("id") == "4":
							for all in wssb.findall("*"):
								wssb.remove(all)
										
							for tall in theme.findall("*"):
								wssb.append(Tree.fromstring(Tree.tostring(tall)))

				else:
					theme.set("value", "inactive")
					
		if rootTheme.find("screenthemes") is not None:
			themes = rootTheme.find("screenthemes")
			for screens in themes.findall("screens"):
				if screens.get("name") == "InfoBar":
					for infobar in screens.findall("screentheme"):
						if infobar.get("name") == config.plugins.MerlinSkinThemes.InfoBar.value:
							infobar.set("value", "active")
							ib = infobar.find("screen")

							# delete old InfoBar screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InfoBar":
									rootSkin.remove(SkinScreen)
							
							# Set new InfoBar screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							infobar.set("value", "inactive")
				
				if screens.get("name") == "ChannelSelection":
					for channelselection in screens.findall("screentheme"):
						if channelselection.get("name") == config.plugins.MerlinSkinThemes.ChannelSelection.value:
							channelselection.set("value", "active")
							cs = channelselection.find("screen")

							# delete old ChannelSelection screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "ChannelSelection":
									rootSkin.remove(SkinScreen)
							
							# Set new ChannelSelection screen
							rootSkin.append(Tree.fromstring(Tree.tostring(cs)))
							
						else:
							channelselection.set("value", "inactive")
				
				if screens.get("name") == "MovieSelection":
					for movieselection in screens.findall("screentheme"):
						if movieselection.get("name") == config.plugins.MerlinSkinThemes.MovieSelection.value:
							movieselection.set("value", "active")
							ms = movieselection.find("screen")

							# delete old MovieSelection screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MovieSelection":
									rootSkin.remove(SkinScreen)
							
							# Set new MovieSelection screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ms)))
								
						else:
							movieselection.set("value", "inactive")
				
				if screens.get("name") == "MoviePlayer":
					for movieplayer in screens.findall("screentheme"):
						if movieplayer.get("name") == config.plugins.MerlinSkinThemes.MoviePlayer.value:
							movieplayer.set("value", "active")
							ms = movieplayer.find("screen")

							# delete old MoviePlayer screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MoviePlayer":
									rootSkin.remove(SkinScreen)
							
							# Set new MoviePlayer screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ms)))
								
						else:
							movieplayer.set("value", "inactive")
				
				if screens.get("name") == "SecondInfoBar":
					for secondinfobar in screens.findall("screentheme"):
						if secondinfobar.get("name") == config.plugins.MerlinSkinThemes.SecondInfoBar.value:
							secondinfobar.set("value", "active")
							sib = secondinfobar.find("screen")

							# delete old SecondInfoBar screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "SecondInfoBar":
									rootSkin.remove(SkinScreen)
							
							# Set new SecondInfoBar screen
							rootSkin.append(Tree.fromstring(Tree.tostring(sib)))
								
						else:
							secondinfobar.set("value", "inactive")
				
				if screens.get("name") == "EventView":
					for eventview in screens.findall("screentheme"):
						if eventview.get("name") == config.plugins.MerlinSkinThemes.EventView.value:
							eventview.set("value", "active")
							sib = eventview.find("screen")

							# delete old EventView screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "EventView":
									rootSkin.remove(SkinScreen)
							
							# Set new EventView screen
							rootSkin.append(Tree.fromstring(Tree.tostring(sib)))
								
						else:
							eventview.set("value", "inactive")
				
				if screens.get("name") == "EPGSelection":
					for epgselection in screens.findall("screentheme"):
						if epgselection.get("name") == config.plugins.MerlinSkinThemes.EPGSelection.value:
							epgselection.set("value", "active")
							sib = epgselection.find("screen")

							# delete old EPGSelection screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "EPGSelection":
									rootSkin.remove(SkinScreen)
							
							# Set new EPGSelection screen
							rootSkin.append(Tree.fromstring(Tree.tostring(sib)))
								
						else:
							epgselection.set("value", "inactive")
				
				if screens.get("name") == "MessageBox":
					for messagebox in screens.findall("screentheme"):
						if messagebox.get("name") == config.plugins.MerlinSkinThemes.MessageBox.value:
							messagebox.set("value", "active")
							mb = messagebox.find("screen")

							# delete old MessageBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MessageBox":
									rootSkin.remove(SkinScreen)
							
							# Set new MessageBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(mb)))
								
						else:
							messagebox.set("value", "inactive")
				
				if screens.get("name") == "InputBox":
					for inputbox in screens.findall("screentheme"):
						if inputbox.get("name") == config.plugins.MerlinSkinThemes.InputBox.value:
							inputbox.set("value", "active")
							ib = inputbox.find("screen")

							# delete old InputBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InputBox":
									rootSkin.remove(SkinScreen)
							
							# Set new InputBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							inputbox.set("value", "inactive")
				
				if screens.get("name") == "ChoiceBox":
					for choicebox in screens.findall("screentheme"):
						if choicebox.get("name") == config.plugins.MerlinSkinThemes.ChoiceBox.value:
							choicebox.set("value", "active")
							ib = choicebox.find("screen")

							# delete old ChoiceBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "ChoiceBox":
									rootSkin.remove(SkinScreen)
							
							# Set new ChoiceBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							choicebox.set("value", "inactive")
				
				if screens.get("name") == "Mute":
					for mute in screens.findall("screentheme"):
						if mute.get("name") == config.plugins.MerlinSkinThemes.Mute.value:
							mute.set("value", "active")
							ib = mute.find("screen")

							# delete old Mute screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "Mute":
									rootSkin.remove(SkinScreen)
							
							# Set new Mute screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							mute.set("value", "inactive")
				
				if screens.get("name") == "Volume":
					for volume in screens.findall("screentheme"):
						if volume.get("name") == config.plugins.MerlinSkinThemes.Volume.value:
							volume.set("value", "active")
							ib = volume.find("screen")

							# delete old Volume screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "Volume":
									rootSkin.remove(SkinScreen)
							
							# Set new Volume screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							volume.set("value", "inactive")

		# LCD
		if rootTheme.find("lcdscreenthemes") is not None:
			themes = rootTheme.find("lcdscreenthemes")
			for screens in themes.findall("screens"):
				if screens.get("name") == "InfoBarSummary":
					for linfobar in screens.findall("lcdscreentheme"):
						if linfobar.get("name") == config.plugins.MerlinSkinThemes.LCDInfoBar.value:
							linfobar.set("value", "active")
							ib = linfobar.find("screen")

							# delete old InfoBarSummary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InfoBarSummary" and SkinScreen.get("id") == "1":
									rootSkin.remove(SkinScreen)
							
							# Set new InfoBarSummary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							linfobar.set("value", "inactive")

				if screens.get("name") == "EventView_summary":
					for leventview in screens.findall("lcdscreentheme"):
						if leventview.get("name") == config.plugins.MerlinSkinThemes.LCDEventView.value:
							leventview.set("value", "active")
							ib = leventview.find("screen")

							# delete old EventView_summary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "EventView_summary" and SkinScreen.get("id") == "1":
									rootSkin.remove(SkinScreen)
							
							# Set new EventView_summary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							leventview.set("value", "inactive")
							
				if screens.get("name") == "StandbySummary":
					for lstandby in screens.findall("lcdscreentheme"):
						if lstandby.get("name") == config.plugins.MerlinSkinThemes.LCDStandby.value:
							lstandby.set("value", "active")
							ib = lstandby.find("screen")

							# delete old StandbySummary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "StandbySummary" and SkinScreen.get("id") == "1":
									rootSkin.remove(SkinScreen)
							
							# Set new StandbySummary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							lstandby.set("value", "inactive")

		# OLED
		if rootTheme.find("oledscreenthemes") is not None:
			themes = rootTheme.find("oledscreenthemes")
			for screens in themes.findall("screens"):
				if screens.get("name") == "InfoBarSummary":
					for oinfobar in screens.findall("oledscreentheme"):
						if oinfobar.get("name") == config.plugins.MerlinSkinThemes.OLEDInfoBar.value:
							oinfobar.set("value", "active")
							ib = oinfobar.find("screen")

							# delete old InfoBarSummary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InfoBarSummary" and SkinScreen.get("id") == "2":
									rootSkin.remove(SkinScreen)
							
							# Set new InfoBarSummary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							oinfobar.set("value", "inactive")

				if screens.get("name") == "EventView_summary":
					for oeventview in screens.findall("oledscreentheme"):
						if oeventview.get("name") == config.plugins.MerlinSkinThemes.OLEDEventView.value:
							oeventview.set("value", "active")
							ib = oeventview.find("screen")

							# delete old EventView_summary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "EventView_summary" and SkinScreen.get("id") == "2":
									rootSkin.remove(SkinScreen)
							
							# Set new EventView_summary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							oeventview.set("value", "inactive")
							
				if screens.get("name") == "StandbySummary":
					for ostandby in screens.findall("oledscreentheme"):
						if ostandby.get("name") == config.plugins.MerlinSkinThemes.OLEDStandby.value:
							ostandby.set("value", "active")
							ib = ostandby.find("screen")

							# delete old StandbySummary screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "StandbySummary" and SkinScreen.get("id") == "2":
									rootSkin.remove(SkinScreen)
							
							# Set new StandbySummary screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							ostandby.set("value", "inactive")

				
		self.XMLindent(rootSkin, 0)
		curSkin.write(MerlinSkinThemes.selSkinFile)
		
		curTree.write(MerlinSkinThemes.selThemeFile)
		
	def createThemes(self):
		if os.path.isfile(MerlinSkinThemes.selThemeFile) == False:
			themes = Tree.Element("themes")


			# colors
			colortheme1 = Tree.SubElement(themes, "colortheme", {"name": "orginal", "value": "active"})
			colorsnode1 = Tree.SubElement(colortheme1, "colors")

			colortheme2 = Tree.SubElement(themes, "colortheme", {"name": "orginal - work", "value": "inactive"})
			colorsnode2 = Tree.SubElement(colortheme2, "colors")

			curTree = Tree.parse(MerlinSkinThemes.selSkinFile)
			colors = curTree.find("colors")

			for color in colors.findall("color"):
				name = color.get("name")
				value = color.get("value")
				
				Tree.SubElement(colorsnode1, "color", {"name": name, "value": value})
				Tree.SubElement(colorsnode2, "color", {"name": name, "value": value})


			# fonts
			fonttheme1 = Tree.SubElement(themes, "fonttheme", {"name": "orginal", "value": "active"})
			fontsnode1 = Tree.SubElement(fonttheme1, "fonts")

			fonttheme2 = Tree.SubElement(themes, "fonttheme", {"name": "orginal - work", "value": "inactive"})
			fontsnode2 = Tree.SubElement(fonttheme2, "fonts")

			fonts = curTree.find("fonts")

			for font in fonts.findall("font"):
				filename = font.get("filename")
				name = font.get("name")
				scale = font.get("scale", "100")
				replacement = font.get("replacement", "0")
				
				Tree.SubElement(fontsnode1, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})
				Tree.SubElement(fontsnode2, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})


			# bordersets
			bordersettheme1 = Tree.SubElement(themes, "bordersettheme", {"name": "orginal", "value": "active"})
			bordersettheme2 = Tree.SubElement(themes, "bordersettheme", {"name": "orginal - work", "value": "inactive"})

			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			ws = rootSkin.find("windowstyle")
			for bs in ws.findall("borderset"):
				if bs.get("name") == "bsWindow":
					bordersettheme1.append(Tree.fromstring(Tree.tostring(bs)))
					bordersettheme2.append(Tree.fromstring(Tree.tostring(bs)))
			
				if bs.get("name") == "bsListboxEntry":
					bordersettheme1.append(Tree.fromstring(Tree.tostring(bs)))
					bordersettheme2.append(Tree.fromstring(Tree.tostring(bs)))

			
			# windowstylescrollbar
			wssbtheme1 = Tree.SubElement(themes, "windowstylescrollbartheme", {"name": "orginal", "value": "active"})
			wssbtheme2 = Tree.SubElement(themes, "windowstylescrollbartheme", {"name": "orginal - work", "value": "inactive"})
			
			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			for wssb in rootSkin.findall("windowstylescrollbar"):
				if wssb.get("id") == "4":
					for sb in wssb.findall("*"):
						wssbtheme1.append(Tree.fromstring(Tree.tostring(sb)))
						wssbtheme2.append(Tree.fromstring(Tree.tostring(sb)))
			
			
			# screenthemes
			screenthemes = Tree.SubElement(themes, "screenthemes")
			infobar = Tree.SubElement(screenthemes, "screens", {"name": "InfoBar"})
			channelselection = Tree.SubElement(screenthemes, "screens", {"name": "ChannelSelection"})
			movieselection = Tree.SubElement(screenthemes, "screens", {"name": "MovieSelection"})
			movieplayer = Tree.SubElement(screenthemes, "screens", {"name": "MoviePlayer"})
			secondinfobar = Tree.SubElement(screenthemes, "screens", {"name": "SecondInfoBar"})
			eventview = Tree.SubElement(screenthemes, "screens", {"name": "EventView"})
			epgselection = Tree.SubElement(screenthemes, "screens", {"name": "EPGSelection"})
			messagebox = Tree.SubElement(screenthemes, "screens", {"name": "MessageBox"})
			inputbox = Tree.SubElement(screenthemes, "screens", {"name": "InputBox"})
			choicebox = Tree.SubElement(screenthemes, "screens", {"name": "ChoiceBox"})
			mute = Tree.SubElement(screenthemes, "screens", {"name": "Mute"})
			volume = Tree.SubElement(screenthemes, "screens", {"name": "Volume"})

			oledscreenthemes = Tree.SubElement(themes, "oledscreenthemes")
			oinfobar = Tree.SubElement(oledscreenthemes, "screens", {"name": "InfoBarSummary", "id": "2"})
			oeventview = Tree.SubElement(oledscreenthemes, "screens", {"name": "EventView_summary", "id": "2"})
			ostandby = Tree.SubElement(oledscreenthemes, "screens", {"name": "StandbySummary", "id": "2"})

			lcdscreenthemes = Tree.SubElement(themes, "lcdscreenthemes")
			linfobar = Tree.SubElement(lcdscreenthemes, "screens", {"name": "InfoBarSummary", "id": "1"})
			leventview = Tree.SubElement(lcdscreenthemes, "screens", {"name": "EventView_summary", "id": "1"})
			lstandby = Tree.SubElement(lcdscreenthemes, "screens", {"name": "StandbySummary", "id": "1"})

			infobar1 = Tree.SubElement(infobar, "screentheme", {"name": "orginal", "value": "active"})
			infobar2 = Tree.SubElement(infobar, "screentheme", {"name": "orginal - work", "value": "inactive"})

			channelselection1 = Tree.SubElement(channelselection, "screentheme", {"name": "orginal", "value": "active"})
			channelselection2 = Tree.SubElement(channelselection, "screentheme", {"name": "orginal - work", "value": "inactive"})

			movieselection1 = Tree.SubElement(movieselection, "screentheme", {"name": "orginal", "value": "active"})
			movieselection2 = Tree.SubElement(movieselection, "screentheme", {"name": "orginal - work", "value": "inactive"})

			movieplayer1 = Tree.SubElement(movieplayer, "screentheme", {"name": "orginal", "value": "active"})
			movieplayer2 = Tree.SubElement(movieplayer, "screentheme", {"name": "orginal - work", "value": "inactive"})

			secondinfobar1 = Tree.SubElement(secondinfobar, "screentheme", {"name": "orginal", "value": "active"})
			secondinfobar2 = Tree.SubElement(secondinfobar, "screentheme", {"name": "orginal - work", "value": "inactive"})

			eventview1 = Tree.SubElement(eventview, "screentheme", {"name": "orginal", "value": "active"})
			eventview2 = Tree.SubElement(eventview, "screentheme", {"name": "orginal - work", "value": "inactive"})

			epgselection1 = Tree.SubElement(epgselection, "screentheme", {"name": "orginal", "value": "active"})
			epgselection2 = Tree.SubElement(epgselection, "screentheme", {"name": "orginal - work", "value": "inactive"})

			messagebox1 = Tree.SubElement(messagebox, "screentheme", {"name": "orginal", "value": "active"})
			messagebox2 = Tree.SubElement(messagebox, "screentheme", {"name": "orginal - work", "value": "inactive"})

			inputbox1 = Tree.SubElement(inputbox, "screentheme", {"name": "orginal", "value": "active"})
			inputbox2 = Tree.SubElement(inputbox, "screentheme", {"name": "orginal - work", "value": "inactive"})

			choicebox1 = Tree.SubElement(choicebox, "screentheme", {"name": "orginal", "value": "active"})
			choicebox2 = Tree.SubElement(choicebox, "screentheme", {"name": "orginal - work", "value": "inactive"})

			mute1 = Tree.SubElement(mute, "screentheme", {"name": "orginal", "value": "active"})
			mute2 = Tree.SubElement(mute, "screentheme", {"name": "orginal - work", "value": "inactive"})

			volume1 = Tree.SubElement(volume, "screentheme", {"name": "orginal", "value": "active"})
			volume2 = Tree.SubElement(volume, "screentheme", {"name": "orginal - work", "value": "inactive"})

			# oled
			oinfobar1 = Tree.SubElement(oinfobar, "oledscreentheme", {"name": "orginal", "value": "active"})
			oinfobar2 = Tree.SubElement(oinfobar, "oledscreentheme", {"name": "orginal - work", "value": "inactive"})

			oeventview1 = Tree.SubElement(oeventview, "oledscreentheme", {"name": "orginal", "value": "active"})
			oeventview2 = Tree.SubElement(oeventview, "oledscreentheme", {"name": "orginal - work", "value": "inactive"})

			ostandby1 = Tree.SubElement(oinfobar, "oledscreentheme", {"name": "orginal", "value": "active"})
			ostandby2 = Tree.SubElement(oinfobar, "oledscreentheme", {"name": "orginal - work", "value": "inactive"})

			# lcd
			linfobar1 = Tree.SubElement(linfobar, "lcdscreentheme", {"name": "orginal", "value": "active"})
			linfobar2 = Tree.SubElement(linfobar, "lcdscreentheme", {"name": "orginal - work", "value": "inactive"})

			leventview1 = Tree.SubElement(leventview, "lcdscreentheme", {"name": "orginal", "value": "active"})
			leventview2 = Tree.SubElement(leventview, "lcdscreentheme", {"name": "orginal - work", "value": "inactive"})

			lstandby1 = Tree.SubElement(linfobar, "lcdscreentheme", {"name": "orginal", "value": "active"})
			lstandby2 = Tree.SubElement(linfobar, "lcdscreentheme", {"name": "orginal - work", "value": "inactive"})
			
			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			for SkinScreen in rootSkin.findall("screen"):
				if SkinScreen.get("name") == "InfoBar":
					infobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					infobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "ChannelSelection":
					channelselection1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					channelselection2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MovieSelection":
					movieselection1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					movieselection2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MoviePlayer":
					movieplayer1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					movieplayer2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					
				if SkinScreen.get("name") == "SecondInfoBar":
					secondinfobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					secondinfobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "EventView":
					eventview1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					eventview2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "EPGSelection":
					epgselection1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					epgselection2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MessageBox":
					messagebox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					messagebox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "InputBox":
					inputbox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					inputbox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "ChoiceBox":
					choicebox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					choicebox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "Mute":
					mute1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					mute2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "Volume":
					volume1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					volume2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				# oled
				if SkinScreen.get("name") == "InfoBarSummary" and SkinScreen.get("id") == "2":
					oinfobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					oinfobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					
				if SkinScreen.get("name") == "EventView_summary" and SkinScreen.get("id") == "2":
					oeventview1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					oeventview2.append(Tree.fromstring(Tree.tostring(SkinScreen)))

				if SkinScreen.get("name") == "StandbySummary" and SkinScreen.get("id") == "2":
					ostandby1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					ostandby2.append(Tree.fromstring(Tree.tostring(SkinScreen)))

				# lcd
				if SkinScreen.get("name") == "InfoBarSummary" and SkinScreen.get("id") == "1":
					linfobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					linfobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					
				if SkinScreen.get("name") == "EventView_summary" and SkinScreen.get("id") == "1":
					leventview1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					leventview2.append(Tree.fromstring(Tree.tostring(SkinScreen)))

				if SkinScreen.get("name") == "StandbySummary" and SkinScreen.get("id") == "1":
					lstandby1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					lstandby2.append(Tree.fromstring(Tree.tostring(SkinScreen)))

			# Sort
			self.XMLindent(themes, 0)
			
			# save xml
			themexml = open(MerlinSkinThemes.selThemeFile, "w")
			themexml.write(Tree.tostring(themes))
			themexml.close()
			
			self.updateSkinsList()
	
	def delSkinDir(self):
		print "[MST171] Delete: %s" % resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/"
		shutil.rmtree(resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/")
		self.updateSkinsList()
		
	def restartGUI(self, answer):
		if answer is True:
			#self.setTheme()
			self.session.open(TryQuitMainloop, 3)
		else:
			self.exit()

	def MsgSkinInfo(self):
		InfoText = "";
		
		curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
		rootSkin = curSkin.getroot()
		if rootSkin.find("copyright") is not None:
			copyright = rootSkin.find("copyright")
			if copyright.find("orginal") is not None:
				org = copyright.find("orginal")

				oAuthor = org.get("author")
				if oAuthor is None:
					oAuthor = ""
				oVersion = org.get("version")
				if oVersion is None:
					oVersion = ""
				oName = org.get("name")
				if oName is None:
					oName = ""
				oSupport = org.get("supporturl")
				if oSupport is None:
					oSupport = ""
				oLicense = org.get("license")
				if oLicense is None:
					oLicense = ""
				
				OrgText = (
					"=== ORGINAL ===\n"
					"Skinname:\t" + oName + "\n" +
					"Author:\t" + oAuthor + "\n" +
					"Version:\t" + oVersion + "\n" +
					"Support:\t" + oSupport + "\n" +
					"License:\n" + oLicense + "\n\n"
				)
			
			else:
				OrgText = (
					"=== ORGINAL ===\n"
					"No copyrightinfo available"
				)
		
			if copyright.find("mod") is not None:
				mod = copyright.find("mod")

				mAuthor = mod.get("author")
				if mAuthor is None:
					mAuthor = ""
				mVersion = mod.get("version")
				if mVersion is None:
					mVersion = ""
				mName = mod.get("name")
				if mName is None:
					mName = ""
				mSupport = mod.get("supporturl")
				if mSupport is None:
					mSupport = ""
				
				ModText = (
					"=== MOD ===\n"
					"Skinname:\t" + mName + "\n" +
					"Author:\t" + mAuthor + "\n" +
					"Version:\t" + mVersion + "\n" +
					"Support:\t" + mSupport
				)
			
			else:
				ModText = (
					"=== MOD ===\n"
					"No copyrightinfo available"
				)
		
			InfoText = OrgText + "\n\n" + ModText

		else:
			InfoText = "No copyrightinfo available"
			
		self.session.open(MessageBox, InfoText, MessageBox.TYPE_INFO, title="About Skin - " + MerlinSkinThemes.selSkinName)
			
	def save(self):
		restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("For apply new skin settings.\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Restart GUI now?"))

	def exit(self):
		print '[MST] closing'
		self["SkinsList"].onSelectionChanged.remove(self.changedSkinsList)
		self.close()
		
	def XMLindent(self, elem, level):
		i = "\n" + (level*"    ")
		#a = "\n%%-%ds" % level
		#i = a % '  '
		
		if len(elem):
			if not elem.text or not elem.text.strip():
				elem.text = i + "    "
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
			for elem in elem:
				self.XMLindent(elem, level+1)
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
		else:
			if level and (not elem.tail or not elem.tail.strip()):
				elem.tail = i
			
def main(session, **kwargs):
	session.open(MerlinSkinThemes)

def Plugins(path,**kwargs):
	list = [PluginDescriptor(name = "MerlinSkinThemes", description = "MerlinSkinThemes", where = PluginDescriptor.WHERE_PLUGINMENU, icon = "plugin.png", fnc = main)]
	return list		

# =================================================================================================

class GetSkinsList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setFont(1, gFont("Regular", 17))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		dirs = os.listdir(resolveFilename(SCOPE_SKIN))
		for dir in dirs:
			if os.path.isdir(resolveFilename(SCOPE_SKIN) + dir) is True:
				curSkinFile = resolveFilename(SCOPE_SKIN) + dir + "/skin.xml"
				curThemeFile = resolveFilename(SCOPE_SKIN) + dir + "/themes.xml"
				
				info = ""
				status = ""

				if os.path.exists(curSkinFile) or os.path.exists(curThemeFile):
					if os.path.exists(curSkinFile) is False:
						info = "no skin.xml"

					if os.path.exists(curThemeFile) is False:
						info = "no themes.xml"

					if dir == SkinName:
						status = "active skin"

					res = [
						dir,
						(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 250, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, dir),
						(eListboxPythonMultiContent.TYPE_TEXT, 255, 0, 155, 24, 1, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
						(eListboxPythonMultiContent.TYPE_TEXT, 405, 0, 155, 24, 1, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, info),
					]
					list.append(res)
				
		self.list = list
		self.l.setList(list)
		
# =================================================================================================

class GetColorThemesList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		for theme in curTree.findall("colortheme"):
			status = ""
			if theme.attrib["value"] == "active":
				status = theme.attrib["value"]
				
			res = [
				theme,
				(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 245, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, theme.attrib["name"]),
				(eListboxPythonMultiContent.TYPE_TEXT, 250, 0, 225, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
				(eListboxPythonMultiContent.TYPE_TEXT, 480, 0, 20, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, ""),
			]
			list.append(res)

		self.list = list
		self.l.setList(list)

# =================================================================================================

class GetFontThemesList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		for theme in curTree.findall("fonttheme"):
			status = ""
			if theme.attrib["value"] == "active":
				status = theme.attrib["value"]
				
			res = [
				theme,
				(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 245, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, theme.attrib["name"]),
				(eListboxPythonMultiContent.TYPE_TEXT, 250, 0, 225, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
				(eListboxPythonMultiContent.TYPE_TEXT, 480, 0, 20, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, ""),
			]
			list.append(res)

		self.list = list
		self.l.setList(list)

# =================================================================================================

class GetScreensList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		#print "[MST] SkinFile: " + SkinFile
		#curTree = Tree.parse(SkinFile)
		#for screens in curTree.findall("screen"):
		#	res = [
		#		screens,
		#		(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, screens.attrib["name"]),
		#	]
		#	list.append(res)

		res = ["", (eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, "InfoBar"),]
		list.append(res)
		
		res = ["", (eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, "ChannelSelection"),]
		list.append(res)

		self.list = list
		self.l.setList(list)
	